module d_webservice_example.dataaccess.in_memory_todo_repository;

import d_webservice_example.business.todo_service : TodoRepository;

version (unittest)
{
    import fluent.asserts;
}

class InMemoryTodoRepository : TodoRepository
{
    import d_webservice_example.model.todo : Todo;
    import optional.optional : Optional;
    import std.uuid : randomUUID, UUID;

private:
    Todo[string] todos;

public:

    this() nothrow pure @nogc @safe
    {
    }

    override Todo save(Todo todo)
    {
        import std.range.primitives : empty;

        if (todo.id.empty)
            todo.id = randomUUID.toString;
        todos[todo.id] = todo;
        return todo;
    }

    unittest
    {
        // given
        auto underTest = new InMemoryTodoRepository;
        auto todo = Todo("some title", "some content",
                UUID("00000000-0000-0000-0000-000000000001"));

        // when
        auto savedTodo = underTest.save(todo);

        // then
        savedTodo.title.should.equal(todo.title);
        savedTodo.content.should.equal(todo.content);
        savedTodo.uuid.should.equal(todo.uuid);
        savedTodo.id.length.should.equal(36);
    }

    unittest
    {
        // given
        auto underTest = new InMemoryTodoRepository;
        auto todo = Todo("some title", "some content",
                UUID("00000000-0000-0000-0000-000000000001"),
                "00000000-0000-0000-0000-000000000002");

        // when
        auto savedTodo = underTest.save(todo);

        // then
        savedTodo.should.equal(todo);
    }

    override Todo[] findAll()
    {
        import std.array : array;

        return todos.byValue.array;
    }

    override Optional!Todo findByUuid(UUID uuid)
    {
        import d_webservice_example.util.algorithm : findFirst;

        return todos.byValue.findFirst!(todo => todo.uuid == uuid);
    }

    override bool remove(Todo todo)
    {
        return todos.remove(todo.id);
    }
}
